@@ -2,7 +2,7 @@
  * dpkg-deb - construction and deconstruction of *.deb archives
  * extract.c - extracting archives
  *
- * Copyright © 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1994,1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -26,10 +26,8 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 
-#include <assert.h>
 #include <errno.h>
 #include <limits.h>
-#include <ctype.h>
 #include <string.h>
 #include <dirent.h>
 #include <fcntl.h>
@@ -109,6 +107,7 @@ extracthalf(const char *debar, const char *dir,
 {
   struct dpkg_error err;
   const char *errstr;
+  struct dpkg_ar *ar;
   char versionbuf[40];
   struct deb_version version;
   off_t ctrllennum, memberlen = 0;
@@ -117,20 +116,14 @@ extracthalf(const char *debar, const char *dir,
   pid_t c1=0,c2,c3;
   int p1[2], p2[2];
   int p2_out;
-  int arfd;
-  struct stat stab;
   char nlc;
   int adminmember = -1;
   bool header_done;
   enum compressor_type decompressor = COMPRESSOR_TYPE_GZIP;
 
-  arfd = open(debar, O_RDONLY);
-  if (arfd < 0)
-    ohshite(_("failed to read archive `%.255s'"), debar);
-  if (fstat(arfd, &stab))
-    ohshite(_("failed to fstat archive"));
+  ar = dpkg_ar_open(debar);
 
-  r = read_line(arfd, versionbuf, strlen(DPKG_AR_MAGIC), sizeof(versionbuf) - 1);
+  r = read_line(ar->fd, versionbuf, strlen(DPKG_AR_MAGIC), sizeof(versionbuf) - 1);
   if (r < 0)
     read_fail(r, debar, _("archive magic version number"));
 
@@ -138,9 +131,9 @@ extracthalf(const char *debar, const char *dir,
     ctrllennum= 0;
     header_done = false;
     for (;;) {
-      struct ar_hdr arh;
+      struct dpkg_ar_hdr arh;
 
-      r = fd_read(arfd, &arh, sizeof(arh));
+      r = fd_read(ar->fd, &arh, sizeof(arh));
       if (r != sizeof(arh))
         read_fail(r, debar, _("archive member header"));
 
@@ -148,14 +141,15 @@ extracthalf(const char *debar, const char *dir,
 
       if (dpkg_ar_member_is_illegal(&arh))
         ohshit(_("file '%.250s' is corrupt - bad archive header magic"), debar);
-      memberlen = dpkg_ar_member_get_size(debar, &arh);
+      memberlen = dpkg_ar_member_get_size(ar, &arh);
       if (!header_done) {
         char *infobuf;
 
         if (strncmp(arh.ar_name, DEBMAGIC, sizeof(arh.ar_name)) != 0)
-          ohshit(_("file `%.250s' is not a debian binary archive (try dpkg-split?)"),debar);
+          ohshit(_("file '%.250s' is not a debian binary archive (try dpkg-split?)"),
+                 debar);
         infobuf= m_malloc(memberlen+1);
-        r = fd_read(arfd, infobuf, memberlen + (memberlen & 1));
+        r = fd_read(ar->fd, infobuf, memberlen + (memberlen & 1));
         if (r != (memberlen + (memberlen & 1)))
           read_fail(r, debar, _("archive information header member"));
         infobuf[memberlen] = '\0';
@@ -175,8 +169,8 @@ extracthalf(const char *debar, const char *dir,
       } else if (arh.ar_name[0] == '_') {
         /* Members with ‘_’ are noncritical, and if we don't understand
          * them we skip them. */
-        if (fd_skip(arfd, memberlen + (memberlen & 1), &err) < 0)
-          ohshit(_("cannot skip archive member from '%s': %s"), debar, err.str);
+        if (fd_skip(ar->fd, memberlen + (memberlen & 1), &err) < 0)
+          ohshit(_("cannot skip archive member from '%s': %s"), ar->name, err.str);
       } else {
         if (strncmp(arh.ar_name, ADMINMEMBER, strlen(ADMINMEMBER)) == 0) {
           const char *extension = arh.ar_name + strlen(ADMINMEMBER);
@@ -217,8 +211,8 @@ extracthalf(const char *debar, const char *dir,
           ctrllennum= memberlen;
         }
         if (!adminmember != !admininfo) {
-          if (fd_skip(arfd, memberlen + (memberlen & 1), &err) < 0)
-            ohshit(_("cannot skip archive member from '%s': %s"), debar, err.str);
+          if (fd_skip(ar->fd, memberlen + (memberlen & 1), &err) < 0)
+            ohshit(_("cannot skip archive member from '%s': %s"), ar->name, err.str);
         } else {
           /* Yes! - found it. */
           break;
@@ -230,7 +224,7 @@ extracthalf(const char *debar, const char *dir,
       printf(_(" new debian package, version %d.%d.\n"
                " size %jd bytes: control archive=%jd bytes.\n"),
              version.major, version.minor,
-             (intmax_t)stab.st_size, (intmax_t)ctrllennum);
+             (intmax_t)ar->size, (intmax_t)ctrllennum);
       m_output(stdout, _("<standard output>"));
     }
   } else if (strncmp(versionbuf, "0.93", 4) == 0) {
@@ -245,19 +239,19 @@ extracthalf(const char *debar, const char *dir,
     if (errstr)
       ohshit(_("archive has invalid format version: %s"), errstr);
 
-    r = read_line(arfd, ctrllenbuf, 1, sizeof(ctrllenbuf) - 1);
+    r = read_line(ar->fd, ctrllenbuf, 1, sizeof(ctrllenbuf) - 1);
     if (r < 0)
       read_fail(r, debar, _("archive control member size"));
-    if (sscanf(ctrllenbuf, "%jd%c%d", &ctrllennum, &nlc, &dummy) != 2 ||
+    if (sscanf(ctrllenbuf, "%jd%c%d", (intmax_t *)&ctrllennum, &nlc, &dummy) != 2 ||
         nlc != '\n')
       ohshit(_("archive has malformatted control member size '%s'"), ctrllenbuf);
 
     if (admininfo) {
       memberlen = ctrllennum;
     } else {
-      memberlen = stab.st_size - ctrllennum - strlen(ctrllenbuf) - l;
-      if (fd_skip(arfd, ctrllennum, &err) < 0)
-        ohshit(_("cannot skip archive control member from '%s': %s"), debar,
+      memberlen = ar->size - ctrllennum - strlen(ctrllenbuf) - l;
+      if (fd_skip(ar->fd, ctrllennum, &err) < 0)
+        ohshit(_("cannot skip archive control member from '%s': %s"), ar->name,
                err.str);
     }
 
@@ -265,8 +259,8 @@ extracthalf(const char *debar, const char *dir,
       printf(_(" old debian package, version %d.%d.\n"
                " size %jd bytes: control archive=%jd, main archive=%jd.\n"),
              version.major, version.minor,
-             (intmax_t)stab.st_size, (intmax_t)ctrllennum,
-             (intmax_t)(stab.st_size - ctrllennum - strlen(ctrllenbuf) - l));
+             (intmax_t)ar->size, (intmax_t)ctrllennum,
+             (intmax_t)(ar->size - ctrllennum - strlen(ctrllenbuf) - l));
       m_output(stdout, _("<standard output>"));
     }
   } else {
@@ -275,16 +269,16 @@ extracthalf(const char *debar, const char *dir,
                " corrupted by being downloaded in ASCII mode"));
     }
 
-    ohshit(_("`%.255s' is not a debian format archive"),debar);
+    ohshit(_("'%.255s' is not a debian format archive"), debar);
   }
 
   m_pipe(p1);
   c1 = subproc_fork();
   if (!c1) {
     close(p1[0]);
-    if (fd_fd_copy(arfd, p1[1], memberlen, &err) < 0)
+    if (fd_fd_copy(ar->fd, p1[1], memberlen, &err) < 0)
       ohshit(_("cannot copy archive member from '%s' to decompressor pipe: %s"),
-             debar, err.str);
+             ar->name, err.str);
     if (close(p1[1]))
       ohshite(_("cannot close decompressor pipe"));
     exit(0);
@@ -307,7 +301,7 @@ extracthalf(const char *debar, const char *dir,
     exit(0);
   }
   close(p1[0]);
-  close(arfd);
+  dpkg_ar_close(ar);
   if (taroption) close(p2[1]);
 
   if (taroption) {
@@ -476,6 +470,9 @@ do_raw_extract(const char *const *argv)
   if (debar == NULL)
     badusage(_("--%s needs .deb filename and directory arguments"),
              cipaction->olong);
+  else if (strcmp(debar, "-") == 0)
+    badusage(_("--%s does not support (yet) reading the .deb from standard input"),
+             cipaction->olong);
 
   dir = *argv++;
   if (dir == NULL)
@@ -486,7 +483,7 @@ do_raw_extract(const char *const *argv)
     badusage(_("--%s takes at most two arguments (.deb and directory)"),
              cipaction->olong);
 
-  m_asprintf(&control_dir, "%s/%s", dir, EXTRACTCONTROLDIR);
+  control_dir = str_fmt("%s/%s", dir, EXTRACTCONTROLDIR);
 
   data_options = DPKG_TAR_EXTRACT | DPKG_TAR_PERMS;
   if (opt_verbose)
