@@ -2,8 +2,8 @@
  * dpkg - main program for package management
  * enquiry.c - status enquiry and listing options
  *
- * Copyright © 1995,1996 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2006,2008-2014 Guillem Jover <guillem@debian.org>
+ * Copyright © 1995,1996 Ian Jackson <ijackson@chiark.greenend.org.uk>
+ * Copyright © 2006, 2008-2015 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -143,7 +143,7 @@ static const struct audit_problem audit_problems[] = {
     "database, they need to be reinstalled:\n")
   }, {
     .check = audit_arch,
-    .value.number = DPKG_ARCH_NONE,
+    .value.number = DPKG_ARCH_EMPTY,
     .explanation = N_("The following packages do not have an architecture:\n")
   }, {
     .check = audit_arch,
@@ -169,7 +169,7 @@ static void describebriefly(struct pkginfo *pkg) {
   l= strlen(pkg->set->name);
   if (l>20) maxl -= (l-20);
 
-  pdesc = pkg_summary(pkg, &pkg->installed, &l);
+  pdesc = pkgbin_summary(pkg, &pkg->installed, &l);
   l = min(l, maxl);
 
   printf(" %-20s %.*s\n", pkg_name(pkg, pnaw_nonambig), l, pdesc);
@@ -273,7 +273,7 @@ unpackchk(const char *const *argv)
 {
   int totalcount, sects;
   struct sectionentry *sectionentries, *se, **sep;
-  struct pkgiterator *it;
+  struct pkgiterator *iter;
   struct pkginfo *pkg;
   const char *thissect;
   char buf[20];
@@ -287,8 +287,8 @@ unpackchk(const char *const *argv)
   totalcount= 0;
   sectionentries = NULL;
   sects= 0;
-  it = pkg_db_iter_new();
-  while ((pkg = pkg_db_iter_next_pkg(it))) {
+  iter = pkg_db_iter_new();
+  while ((pkg = pkg_db_iter_next_pkg(iter))) {
     if (!yettobeunpacked(pkg, &thissect)) continue;
     for (se= sectionentries; se && strcasecmp(thissect,se->name); se= se->next);
     if (!se) {
@@ -304,27 +304,27 @@ unpackchk(const char *const *argv)
     }
     se->count++; totalcount++;
   }
-  pkg_db_iter_free(it);
+  pkg_db_iter_free(iter);
 
   if (totalcount == 0)
     return 0;
 
   if (totalcount <= 12) {
-    it = pkg_db_iter_new();
-    while ((pkg = pkg_db_iter_next_pkg(it))) {
+    iter = pkg_db_iter_new();
+    while ((pkg = pkg_db_iter_next_pkg(iter))) {
       if (!yettobeunpacked(pkg, NULL))
         continue;
       describebriefly(pkg);
     }
-    pkg_db_iter_free(it);
+    pkg_db_iter_free(iter);
   } else if (sects <= 12) {
     for (se= sectionentries; se; se= se->next) {
       sprintf(buf,"%d",se->count);
       printf(_(" %d in %s: "),se->count,se->name);
       width= 70-strlen(se->name)-strlen(buf);
       while (width > 59) { putchar(' '); width--; }
-      it = pkg_db_iter_new();
-      while ((pkg = pkg_db_iter_next_pkg(it))) {
+      iter = pkg_db_iter_new();
+      while ((pkg = pkg_db_iter_next_pkg(iter))) {
         const char *pkgname;
 
         if (!yettobeunpacked(pkg,&thissect)) continue;
@@ -335,7 +335,7 @@ unpackchk(const char *const *argv)
         if (width < 4) { printf(" ..."); break; }
         printf(" %s", pkgname);
       }
-      pkg_db_iter_free(it);
+      pkg_db_iter_free(iter);
       putchar('\n');
     }
   } else {
@@ -458,7 +458,7 @@ predeppackage(const char *const *argv)
 {
   static struct varbuf vb;
 
-  struct pkgiterator *it;
+  struct pkgiterator *iter;
   struct pkginfo *pkg = NULL, *startpkg, *trypkg;
   struct dependency *dep;
   struct deppossi *possi, *provider;
@@ -471,8 +471,8 @@ predeppackage(const char *const *argv)
   clear_istobes();
 
   dep = NULL;
-  it = pkg_db_iter_new();
-  while (!dep && (pkg = pkg_db_iter_next_pkg(it))) {
+  iter = pkg_db_iter_new();
+  while (!dep && (pkg = pkg_db_iter_next_pkg(iter))) {
     /* Ignore packages user doesn't want. */
     if (pkg->want != PKG_WANT_INSTALL)
       continue;
@@ -490,7 +490,7 @@ predeppackage(const char *const *argv)
     pkg->clientdata->istobe = PKG_ISTOBE_NORMAL;
     /* If dep is NULL we go and get the next package. */
   }
-  pkg_db_iter_free(it);
+  pkg_db_iter_free(iter);
 
   if (!dep)
     return 1; /* Not found. */
@@ -574,14 +574,6 @@ printarch(const char *const *argv)
   return 0;
 }
 
-int
-printinstarch(const char *const *argv)
-{
-  warning(_("obsolete option '--%s'; please use '--%s' instead"),
-          "print-installation-architecture", "print-architecture");
-  return printarch(argv);
-}
-
 int
 print_foreign_arches(const char *const *argv)
 {
@@ -612,6 +604,7 @@ cmpversions(const char *const *argv)
     /* These values are exit status codes, so 0 = true, 1 = false. */
     int if_lesser, if_equal, if_greater;
     int if_none_a, if_none_both, if_none_b;
+    bool obsolete;
   };
 
   static const struct relationinfo relationinfos[]= {
@@ -630,11 +623,11 @@ cmpversions(const char *const *argv)
     { "gt-nl",     1,1,0, 0,1,1  },
 
     /* For compatibility with dpkg control file syntax. */
-    { "<",         0,0,1, 0,0,1  },
+    { "<",         0,0,1, 0,0,1, .obsolete = true },
     { "<=",        0,0,1, 0,0,1  },
     { "<<",        0,1,1, 0,1,1  },
     { "=",         1,0,1, 1,0,1  },
-    { ">",         1,0,0, 1,0,0  },
+    { ">",         1,0,0, 1,0,0, .obsolete = true },
     { ">=",        1,0,0, 1,0,0  },
     { ">>",        1,1,0, 1,1,0  },
     { NULL                       }
@@ -653,12 +646,13 @@ cmpversions(const char *const *argv)
 
   if (!rip->string) badusage(_("--compare-versions bad relation"));
 
+  if (rip->obsolete)
+    warning(_("--%s used with obsolete relation operator '%s'"),
+            cipaction->olong, rip->string);
+
   if (*argv[0] && strcmp(argv[0],"<unknown>")) {
     if (parseversion(&a, argv[0], &err) < 0) {
-      if (err.type == DPKG_MSG_WARN)
-        warning(_("version '%s' has bad syntax: %s"), argv[0], err.str);
-      else
-        ohshit(_("version '%s' has bad syntax: %s"), argv[0], err.str);
+      dpkg_error_print(&err, _("version '%s' has bad syntax"), argv[0]);
       dpkg_error_destroy(&err);
     }
   } else {
@@ -666,10 +660,7 @@ cmpversions(const char *const *argv)
   }
   if (*argv[2] && strcmp(argv[2],"<unknown>")) {
     if (parseversion(&b, argv[2], &err) < 0) {
-      if (err.type == DPKG_MSG_WARN)
-        warning(_("version '%s' has bad syntax: %s"), argv[2], err.str);
-      else
-        ohshit(_("version '%s' has bad syntax: %s"), argv[2], err.str);
+      dpkg_error_print(&err, _("version '%s' has bad syntax"), argv[2]);
       dpkg_error_destroy(&err);
     }
   } else {
